#!/system/bin/sh
MODDIR=${0%/*}

# Paths
BASEDIR=/data/adb/modules/APreload
INT=/storage/emulated/0
RWD=$INT/APreload
MSC=$BASEDIR/preload
SEARCH_PATHS2="/vendor/lib/egl"
SEARCH_PATHS3="/vendor/lib/hw"

# Notification Function
notification() {
    su -lp 2000 -c "cmd notification post -S bigtext -t '$1' tag '$2'" >/dev/null 2>&1
}

# Ensure directories and files exist
[ ! -d "$RWD" ] && mkdir -p "$RWD"
[ ! -e "$RWD/applist_perf.txt" ] && cp -f "$MSC/applist_perf.txt" "$RWD"

# Get total RAM in GB
total_ram() {
    grep MemTotal /proc/meminfo | awk '{print $2}' | awk '{print ($1 + 1048576) / 1048576}'
}

# Run one-time OpenGL/Vulkan library handling
{
    find "$SEARCH_PATHS2" -type f 2>/dev/null | while read -r file; do
        if strings "$file" | grep -Eo 'libEGL_mtk.so|libGLESv1_CM_mtk.so|libGLESv2_mtk.so|libGLES_meow.so|libMEOW_data.so|libMEOW_gift.so|libMEOW_trace.so|eglSubDriverAndroid.so|libQTapGLES.so|libMEOW_qt.so|libEGL_adreno.so|libGLESv1_CM_adreno.so|libGLESv2_adreno.so|libq3dtools_adreno.so|libq3dtools_esx.so|libVkLayer_ADRENO_qprofiler.so|libEGL_POWERVR_ROGUE.so|libGLESv1_CM_POWERVR_ROGUE.so|libGLESv2_POWERVR_ROGUE.so|libGLES_mali.so' >/dev/null; then
            vmt -dL "$file"
        fi
    done

    find "$SEARCH_PATHS3" -type f 2>/dev/null | while read -r file; do
        if strings "$file" | grep -Eo 'vulkan.universal7904.so|vulkan.mali.so|vulkan.universal9825_r.so|vulkan.mt6765.so|vulkan.adreno.so' >/dev/null; then
            vmt -dL "$file"
        fi
    done
} &

# Update description and notify
notification "Preload Auto" "Waiting for 1 Minute"
sleep 10
pkill -f vmt
notification "Preload Auto" "Preload Checking Successful"

# Initialize state variable
current_state="stopped"

# Main AI Loop
while true; do
    sleep 30

    # Build app filter list
    app_list_filter="grep -o -e applist.app.add"
    while IFS= read -r applist || [[ -n "$applist" ]]; do
        filter=$(echo "$applist" | awk '!/ /')
        [[ -n "$filter" ]] && app_list_filter+=" -e $filter"
    done < "$RWD/applist_perf.txt"

    # Check active app
    window=$(dumpsys window | grep package | $app_list_filter | tail -1)
    if [[ -n "$window" ]]; then
        if [[ "$current_state" != "running" ]]; then
            pkill -f vmt
            package=$(echo "$window" | awk '{print $NF}')
            MEM_GB=$(total_ram)

            if (( $(echo "$MEM_GB < 5" | bc -l) )); then
                vmt -dL "$(cmd package path android | cut -f2 -d:)"
                vmt -dL "$(cmd package path com.android.systemui | cut -f2 -d:)"
            else
                find "$SEARCH_PATHS2" -type f 2>/dev/null | while read -r file; do
                    if strings "$file" | grep -Eo 'libEGL_mtk.so|libGLESv1_CM_mtk.so|libGLESv2_mtk.so|libGLES_meow.so|libMEOW_data.so|libMEOW_gift.so|libMEOW_trace.so|eglSubDriverAndroid.so|libQTapGLES.so|libMEOW_qt.so|libEGL_adreno.so|libGLESv1_CM_adreno.so|libGLESv2_adreno.so|libq3dtools_adreno.so|libq3dtools_esx.so|libVkLayer_ADRENO_qprofiler.so|libEGL_POWERVR_ROGUE.so|libGLESv1_CM_POWERVR_ROGUE.so|libGLESv2_POWERVR_ROGUE.so|libGLES_mali.so' >/dev/null; then
                        vmt -dL "$file"
                    fi
                done

                find "$SEARCH_PATHS3" -type f 2>/dev/null | while read -r file; do
                    if strings "$file" | grep -Eo 'vulkan.universal7904.so|vulkan.mali.so|vulkan.universal9825_r.so|vulkan.mt6765.so|vulkan.adreno.so' >/dev/null; then
                        vmt -dL "$file"
                    fi
                done

                for al in $(find "$(cmd package path "$package" | head -n1 | cut -f2 -d: | xargs -n1 dirname)/lib" -type f); do
                    if strings "$al" | grep -Eo 'libunity.so|libmain.so|libUE4.so|libframeestimationVK.so|libframeestimationGL.so|libvirglrenderer.so|libvortekrenderer.so|libwinlator.so|libminecraftpe.so'; then
                        vmt -dL "$al"
                    fi
                done
            fi

            sed -Ei "s/^description=(\[.*][[:space:]]*)?/description=[ ⚡ Running $window ] /g" "$BASEDIR/module.prop"
            notification "Preload Auto" "Preload Running $window ..."
            current_state="running"
        fi
        sleep 20
    else
        if [[ "$current_state" != "stopped" ]]; then
            pkill -f vmt
            sed -Ei "s/^description=(\[.*][[:space:]]*)?/description=[ ⛔ STOPPED ] /g" "$BASEDIR/module.prop"
            notification "Preload Auto" "Preload Stopped"
            current_state="stopped"
        fi
    fi
done

exit 0
